package com.gic.auth.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuthItemListDTO;
import com.gic.auth.dto.MenuAuthDepartDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.qo.AuthItemSaveQO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import com.gic.auth.qo.UserMenuQO;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 权限树
 * @ClassName: MenuApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/9/17 10:44 AM 
 */
public interface MenuApiService {
    /**
     * 根据list id查询
     * @Title: listByMenuIdList 
     * @Description:
      * @author guojuxing
     * @param menuIdList 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listByMenuIdList(List<Integer> menuIdList);

    ServiceResponse<List<MenuDTO>> listByParentMenuIdList(List<Integer> parentMenuIdList);

    /**
     * 普通管理员（非超级管理员）查询所拥有的应用权限
     * @Title: getAppIdSetByUserId 
     * @Description:
     * @author guojuxing
     * @param userId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.Set<java.lang.String>>  应用ID集合
     */
    ServiceResponse<Set<String>> getAppIdSetByUserId(Integer userId);

    /**
     * 查询用户拥有的所有菜单路由权限，用于登录用户session用
     * @Title: listUserMenu 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @param versionCode 商户服务订购的版本
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listUserMenu(Integer userId, String versionCode);

    /**
     * 获取登陆用户的菜单(页面) 已构建成树结构
     * @param enterpriseId
     * @param userId
     * @param currentMenu
     * @param versionCode 商户服务订购的版本
     * @return
     */
    ServiceResponse<List<MenuDTO>> getUserMenuOfGic(Integer enterpriseId, Integer userId, Integer currentMenu,
            String versionCode);

    ServiceResponse<List<MenuDTO>> getUserMenuOfGic(UserMenuQO params);

    /**
     * 获取登陆用户的菜单(页面)
     * @Title: getUserMenuOfGicNotTree 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param userId
     * @param currentMenu
     * @param versionCode 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<List<MenuDTO>> getUserMenuOfGicNotTree(Integer enterpriseId, Integer userId, Integer currentMenu,
                                                    String versionCode);

    /**
     * 获取协作应用的权限页面
     * @Title: getCollaboratorAppMenuPage 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param menuIdList
     * @param appId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<List<MenuDTO>> getCollaboratorAppMenuPage(Integer enterpriseId, List<Integer> menuIdList, String appId);


    /**
     * 获取协作应用的没有权限的操作项列表数据
     * @Title: getCollaboratorAppOperationItem 
     * @Description:
     * @author guojuxing
     * @param menuIdList 协作应用权限
     * @param appId 应用ID
     * @param  userId 管理员
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<List<MenuDTO>> getCollaboratorAppOperationItem(List<Integer> menuIdList, String appId, Integer userId);

    /**
     * 获取登陆用户的应用菜单（页面类型）已构建成树结构
     * @Title: getUserMenuOfApp 
     * @Description: 
     * @author guojuxing 
     * @param enterpriseId
     * @param userId
     * @param currentMenu
     * @param versionCode 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<List<MenuDTO>> getUserMenuOfApp(Integer enterpriseId, Integer userId, Integer currentMenu,
            String versionCode);

    ServiceResponse<List<MenuDTO>> getUserMenuOfAppNotTree(Integer enterpriseId, Integer userId, Integer currentMenu,
                                                    String versionCode);

    /**
     * 根据用户ID查询没拥有的gic操作项（gic）
     * @Title: getNoAuthOperationItemOfGic 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @param versionCode 商户服务订购的版本
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> getNoAuthOperationItemOfGic(Integer userId, String versionCode);
    /**
     * 根据用户ID查询没拥有的gic操作项(应用)
     * @Title: getNoAuthOperationItemOfApp 
     * @Description:
     * @author guojuxing
     * @param userId
     * @param versionCode 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<List<MenuDTO>> getNoAuthOperationItemOfApp(Integer userId, String versionCode);

    /**
     * 获取超级管理员的Gic菜单
     * @param enterpriseId
     * @return
     */
    ServiceResponse<List<MenuDTO>> getSuperAdminGicMenu(Integer enterpriseId);

    /**
     * 获取超级管理员的菜单 非树    用于登录用户session用
     * @Title: getSuperAdminGicMenuNotTree 
     * @Description:
      * @author guojuxing
     * @param versionCode 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> getAllMenuNotTree(String versionCode);

    /**
     * 查询gic菜单页面树数据
     * @Title: listMenuTree 
     * @Description:
      * @author guojuxing  
     * @param params
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listMenuTree(MenuListQO params);

    ServiceResponse<List<MenuAuthDepartDTO>> listAllAuthDepart(List<String> menuCodeList);

    /**
     * 查询菜单树（页面&权限项）
     * @Title: listMenuTreeForRole 
     * @Description:
      * @author guojuxing
     * @param project 
     * @param versionCode gic商户基础服务版本
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listMenuTreeForRole(String project, String versionCode);

    /**
     * 查询菜单树（页面&权限项）
     * @Title: listMenuTreeForRoleOfGic 
     * @Description:
      * @author guojuxing  
     * @param versionCode gic商户基础服务版本
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listMenuTreeForRoleOfGic(String versionCode);

    /**
     * 查询页面下面的操作项列表数据
     * @Title: listMenuItemByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<List<MenuDTO>> listMenuItemByParentId(Integer parentId);

    /**
     * 新增页面
     * @Title: saveGICPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> saveGICPage(MenuDTO menuDTO);

    /**
     * 查询已经购买的应用
     * @Title: listAppPageOfHasBuy 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param userId 登录用户的ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<List<MenuDTO>> listAppPageOfHasBuy(Integer enterpriseId, Integer userId);

    /**
     * 查询子管理员应用菜单权限数据
     * @Title: listAppPageOfHasBuyForRole 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<List<MenuDTO>> listAppPageOfHasBuyForRole(Integer enterpriseId);

    /**
     * 编辑gic页面
     * @Title: updateGICPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateGICPage(MenuDTO menuDTO);

    /**
     * 删除页面、子页面、权限项、操作项
     * @Title: deleteMenu 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> deleteMenu(Integer menuId);

    /**
     * 新增GIC子页面
     * @Title: saveGICChildPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveGICChildPage(MenuDTO menuDTO);

    /**
     * 编辑gic子页面
     * @Title: updateGICChildPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateGICChildPage(MenuDTO menuDTO);

    /**
     * 编辑应用页面
     * @Title: updateAppPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateAppPage(MenuDTO menuDTO);

    /**
     * 新增应用子页面
     * @Title: saveAppChildPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveAppChildPage(MenuDTO menuDTO);

    /**
     * 编辑应用子页面
     * @Title: updateAppChildPage 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateAppChildPage(MenuDTO menuDTO);

    /**
     * 拖拽排序
     * @Title: setSort 
     * @Description:
      * @author guojuxing
     * @param id
     * @param sort 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> setSort(Integer id, Integer sort);

    /**
     * 新增操作项
     * @Title: saveOperationItem 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveOperationItem(MenuDTO menuDTO);

    /**
     * 编辑操作项
     * @Title: updateOperationItem 
     * @Description:
      * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateOperationItem(MenuDTO menuDTO);

    /**
     * 分页查询操作项
     * @Title: pageMenuOperationItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.auth.dto.MenuDTO>> 
      */
    ServiceResponse<Page<MenuDTO>> pageMenuOperationItem(MenuOperationItemListQO params);

    /**
     * 获取详情
     * @Title: getDetailById 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.MenuDTO> 
      */
    ServiceResponse<MenuDTO> getDetailById(Integer menuId);

    /**
     * 新增权限项
     * @Title: saveAuthItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> saveAuthItem(AuthItemSaveQO params);

    /**
     * 编辑权限项
     * @Title: updateAuthItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateAuthItem(AuthItemSaveQO params);

    /**
     * 查询页面下的权限项
     * @Title: listAuthItemByMenuId 
     * @Description:
      * @author guojuxing
     * @param menuId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AuthItemListDTO>> 
      */
    ServiceResponse<List<AuthItemListDTO>> listAuthItemByMenuId(Integer menuId);

    /**
     * 获取gic基础服务的版本列表数据
     * @Title: getGicDefaultVersion 
     * @Description:
      * @author guojuxing  
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.String,java.lang.String>> 
      */
    ServiceResponse<Map<String, String>> getGicDefaultVersion();

    /**
     * 根据code查询，因为code唯一
     * @Title: getMenuByMenuCode 
     * @Description:
     * @author guojuxing
     * @param menuCode 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.MenuDTO> 
     */
    ServiceResponse<MenuDTO> getMenuByMenuCode(String menuCode);

    /**
     * 同步测试菜单数据到生产
     * @Title: syncProductMenu 
     * @Description:
     * @author guojuxing
     * @param menuCode 第一层级菜单code
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> syncProductMenu(String menuCode);

    ServiceResponse<List<MenuDTO>> test(String menuCode);

    /**
     * 过滤运维用户部门职位权限
     * @param resultList
     * @param tabSysUser
     * @return
     */
    ServiceResponse<List<MenuDTO>> filterOperationUserDepartAuth(List<MenuDTO> resultList, UserDTO tabSysUser);
}
